///////////////////////////////////////////////////////////////////////////////
//                                                                           //
// Class: UD.uc                                                              //
// Author: UD Team                                                           //
// Date: February 2004 - May 2005                                            //
// Purpose: This class handles the gametype functionality for the "Unreal    //
// Demolition" gametype, also known as "Classic Demolition".                 //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////
class UD extends xDeathMatch;

#exec OBJ LOAD FILE=UDHUD.utx

var(LoadingHints) private localized array<string> UDHints;

///////////////////////////////////////////////////////////
// Function to handle respawning of player on death.     //
///////////////////////////////////////////////////////////
function RestartPlayer( Controller aPlayer )
{
    Super.RestartPlayer(aPlayer);
    SpawnVehicle(aPlayer.Pawn);
}

///////////////////////////////////////////////////////////
// Function to handle spawning of vehicle. In the future //
// this function will handle the user's selected vehicle.//
///////////////////////////////////////////////////////////
function SpawnVehicle(Pawn P)
{
    local Vehicle EntryVehicle, V;

    if (P == none)
       return;

    //Used to hold data for spawning multiple vehicles.
    //Erased for time being until multi vehicles are replaced.
    if (P.PlayerReplicationInfo != None)
    {
        V = spawn(class'UDBike',,,P.Location, P.Rotation);
    }
    else // handles case if no PRI
        V = spawn(class'UDBike',,,P.Location, P.Rotation);

    //If no vehicle is found, this portion of the code forces a vehicle
    if (V != none)
    {
        // Sets the PreviousVehicleClass, needed to check what vehicle the player had last time
        if (P.PlayerReplicationInfo != None)
            UDPRI(P.PlayerReplicationInfo).PreviousVehicleClass = V.class;

        if (PlayerController(P.Controller) != None)
        {
            if (PlayerController(P.Controller).bBehindView == false)
            {
                PlayerController(P.Controller).ToggleBehindView();
                PlayerController(P.Controller).BehindView(True);
            }
        }

        // Copied code from PlayerController.ServerUse()
        EntryVehicle = V.FindEntryVehicle(P);
        if (EntryVehicle != None && EntryVehicle.TryToDrive(P))
            return;

        V.UsedBy(P);
        if ( P.Base != None )
            P.Base.UsedBy(P);

        P.Health-=1;
        P.Health+=1;
    }
    else
        log ("BUG: NO VEHICLE!");
}

///////////////////////////////////////////////////////////
// This function handles the spawning of bots based on   //
// the UDBot class of bots.                              //
///////////////////////////////////////////////////////////
function Bot SpawnBot(optional string botName)
{
    local Bot NewBot;
    local RosterEntry Chosen;
    local UnrealTeamInfo BotTeam;

    BotTeam = GetBotTeam();
    Chosen = BotTeam.ChooseBotClass(botName);

    if (Chosen.PawnClass == None)
        Chosen.Init(); //amb
    NewBot = Spawn(Class'UDBot');

    if ( NewBot != None )
        InitializeBot(NewBot,BotTeam,Chosen);
    return NewBot;
}

///////////////////////////////////////////////////////////
// Handles exiting of the game or server.                //
///////////////////////////////////////////////////////////
function Logout(Controller Exiting)
{
    local UDVech DrivenVehicle;

    if (Exiting.Pawn != None)
        DrivenVehicle = UDVech(Exiting.Pawn);

    if ( DrivenVehicle != None )
    {
        DrivenVehicle.KDriverLeave(true); // Force the driver out of the car
        DrivenVehicle.Destroy();
    }
    Super.Logout(Exiting);
}

///////////////////////////////////////////////////////////
// Handles functionality when a player is either killed  //
// or commits suicide.  Functionality includes auto      //
// taunts, score checking, and score adjustments.        //
///////////////////////////////////////////////////////////
function ScoreKill(Controller Killer, Controller Other)
{
    local PlayerReplicationInfo OtherPRI;

    OtherPRI = Other.PlayerReplicationInfo;
    if ( OtherPRI != None )
    {
        OtherPRI.NumLives++;
        if ( (MaxLives > 0) && (OtherPRI.NumLives >=MaxLives) )
            OtherPRI.bOutOfLives = true;
    }

    if ( bAllowTaunts && (Killer != None) && (Killer != Other) && Killer.AutoTaunt()
        && (Killer.PlayerReplicationInfo != None) && (Killer.PlayerReplicationInfo.VoiceType != None) )
    {
        if( Killer.IsA('PlayerController') )
        {
            Killer.SendMessage(OtherPRI, 'AUTOTAUNT', Killer.PlayerReplicationInfo.VoiceType.static.PickRandomTauntFor(Killer, false, false), 10, 'GLOBAL');
        }
        else
        {
            Killer.SendMessage(OtherPRI, 'AUTOTAUNT', Killer.PlayerReplicationInfo.VoiceType.static.PickRandomTauntFor(Killer, false, true), 10, 'GLOBAL');
        }
    }

    // suicide
    if ( (Killer == Other) || (Killer == none) )
    {
        Killer.PlayerReplicationInfo.Score -= 1;
        ScoreEvent(Other.PlayerReplicationInfo, -1, "self_frag");
    }
    else if (Killer.PlayerReplicationInfo != none)
    {
        Killer.PlayerReplicationInfo.Kills++;
        ScoreEvent(Killer.PlayerReplicationInfo, 2, "frag");
        Killer.PlayerReplicationInfo.Score += 2;
    }
    if (gameRulesModifiers != none)
    {
        gameRulesModifiers.scoreKill(Killer, Other);
    }

    if ( (Killer != None) || (MaxLives > 0) )
    {
        CheckScore(Killer.PlayerReplicationInfo);
    }

    if ( (killer == None) || (Other == None) )
    {
        return;
    }

    if ( bAdjustSkill && (killer.IsA('PlayerController') || Other.IsA('PlayerController')) )
    {
        if ( killer.IsA('AIController') )
            AdjustSkill(AIController(killer), PlayerController(Other),true);
        if ( Other.IsA('AIController') )
            AdjustSkill(AIController(Other), PlayerController(Killer),false);
    }
}

///////////////////////////////////////////////////////////
// Overrides old NotifySpree from DeathMatch.uc to allow //
// for killing spree bonus. I copied the entire function //
// due to necessity for section of code to be in a very  //
// precise spot in the block.                            //
///////////////////////////////////////////////////////////
function NotifySpree(Controller Other, int num)
{
	local Controller C;

	if ( num == 5 )
		num = 0;
	else if ( num == 10 )
		num = 1;
	else if ( num == 15 )
		num = 2;
	else if ( num == 20 )
		num = 3;
	else if ( num == 25 )
		num = 4;
	else if ( num == 30 )
		num = 5;
	else
		return;

	SpecialEvent(Other.PlayerReplicationInfo,"spree_"$(num+1));
	if ( TeamPlayerReplicationInfo(Other.PlayerReplicationInfo) != None )
	{
		TeamPlayerReplicationInfo(Other.PlayerReplicationInfo).Spree[num] += 1;
		if ( num > 0 )
			TeamPlayerReplicationInfo(Other.PlayerReplicationInfo).Spree[num-1] -= 1;
	}
	Other.AwardAdrenaline( ADR_MajorKill );

	for ( C=Level.ControllerList; C!=None; C=C.NextController )
		if ( PlayerController(C) != None )
		{
			PlayerController(C).ReceiveLocalizedMessage( class'KillingSpreeMessage', Num, Other.PlayerReplicationInfo );
		    if (Other.PlayerReplicationInfo != none)
            {
                ScoreEvent(Other.PlayerReplicationInfo, 5, "frag");
                Other.PlayerReplicationInfo.Score += 5;
            }
		}
}

///////////////////////////////////////////////////////////
// Loads hints for loading screen.                       //
///////////////////////////////////////////////////////////
static function array<string> GetAllLoadHints(optional bool bThisClassOnly)
{
    local int i;
    local array<string> Hints;

    for ( i = 0; i < default.UDHints.Length; i++ )
        Hints[Hints.Length] = default.UDHints[i];

    return Hints;
}

///////////////////////////////////////////////////////////
// Default properties for the UD gametype.               //
///////////////////////////////////////////////////////////
defaultproperties
{
     //Custom Hints
     UDHints(0)="Press %LEFT% or %RIGHT% to make your vehicle spin in mid-air."
     UDHints(1)="Press %CROUCH% or %JUMP% to make your vehicle flip in mid-air."
     UDHints(2)="Press [Roll Left] or [Roll right] to perform barrel rolls."
     UDHints(3)="Press %JUMP% while on the ground to activate the handbrake."
     UDHints(4)="Press %FIRE% to perform a jump. while in the air %FIRE% will make your vehicle stomp back to the ground."
     UDHints(5)="Press %NEXTWEAPON% and %PREVWEAPON% to control the vehicle's camera zoom."
     UDHints(6)="Pressing %ALTFIRE% will give you a sudden burst of speed."
     UDHints(7)="Jumping and Boosting uses up your PSI. If you run out you'll be sorry."
     UDHints(8)="Pick up PSI and Armor pickups to improve your vehicle and live longer."
     UDHints(9)="%ALTFIRE% will flip your vehicle if you land on your back."
     UDHints(10)="Pressing %ALTFIRE% repeatedly while your vehicle isn't upright will make your vehicle dance."
     UDHints(11)="You can set what keys you can perform barrel rolls with in your input settings."
     UDHints(12)="Killing other players will give you bonus points."
     UDHints(13)="Trying to get too far out of a map can result in the blowing up of your vehicle."

     bAllowVehicles=True
     ScoreBoardType="UnrealDemolition.UDScoreBoard"
     HUDType="UnrealDemolition.UDHUD"
     MapPrefix="UD"
     BeaconName="UD"
     GoalScore=45
     DeathMessageClass=Class'UnrealDemolition.UDDeathMessage'
     PlayerControllerClassName="UnrealDemolition.UDPlayer"
     GameName="Classic Demolition"
     Description="The classic sport of Demolition Derby brought to your living room. Battle in a intense free for all in unique arenas built for serious combat and insane tricks."
     ScreenShotName="UDHUD.UDHUD.intro.uddmintro"
     Acronym="UD"
}
